package edu.toronto.cs.ctl;

import edu.toronto.cs.binarytree.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.algebra.*;
import java.io.*;
import java.util.*;
import edu.toronto.cs.ctl.parser.*;

/**
 ** A converter that takes a binary tree constructed wuth BTNode and
 ** creates a corresponding tree using CTLNode.
 **/
public class BinaryTreeToCTLConverter
{

  static FiniteLattice lattice = null;

  static public void AssignLattice (FiniteLattice fl)
  {
    lattice = fl;
  }

  /**
   ** Converts a string into a CTL parse tree.
   **/
  static public CTLNode convertToCTL (String tree)
  {
    return convertToCTL (tree, new LinkedList ());
  }
  
  static public CTLNode convertToCTL (String tree, List fairness)  
  {
    CTLParser parser = new CTLParser (new StringBufferInputStream (tree));
    BTNode bintree = parser.parseToBTNode ();

    return BinaryTreeToCTLConverter.convertToCTL (bintree, fairness);
  }

  /**
   ** Converts a binary tree into a CTL parse tree.
   **/
  static public CTLNode convertToCTL (BTNode tree)
  {
    return convertToCTL (tree, new LinkedList ());
  }

  static public CTLNode convertToCTL (BTNode tree, List fairness)  
  {
    String data = (String)tree.getData ();

    if (data.equals ("!"))
      return CTLFactory.createCTLNegNode (convertToCTL (tree.getRight (),
							fairness));
    else if (data.equals ("/\\"))
      return CTLFactory.createCTLAndNode (convertToCTL (tree.getLeft (),
							fairness),
					  convertToCTL (tree.getRight (),
							fairness));
    else if (data.equals ("\\/"))
      return CTLFactory.createCTLOrNode (convertToCTL (tree.getLeft (),
						       fairness),
					 convertToCTL (tree.getRight (),
						       fairness));
    else if (data.equals ("->"))
      return CTLFactory.createCTLImplNode (convertToCTL (tree.getLeft (),
							 fairness),
					   convertToCTL (tree.getRight (),
							 fairness));
    else if (data.equals ("<-"))
      return CTLFactory.createCTLImplNode (convertToCTL (tree.getRight (),
							 fairness),
					   convertToCTL (tree.getLeft (),
							 fairness));
    else if (data.equals ("<->"))
      return CTLFactory.createCTLIffNode (convertToCTL (tree.getLeft (),
							fairness),
					  convertToCTL (tree.getRight (),
							fairness));
    else if (data.equals ("="))
      return CTLFactory.createCTLEqualsNode (convertToCTL (tree.getLeft (),
							   fairness),
					     convertToCTL (tree.getRight (),
							   fairness));
    else if (data.equals ("!="))
      return CTLFactory.createCTLEqualsNode (convertToCTL (tree.getLeft (),
							   fairness),
					     convertToCTL (tree.getRight (),
							   fairness)).neg ();
    else if (data.equals ("<="))
      return CTLFactory.createCTLUnderNode (convertToCTL (tree.getLeft (),
							  fairness),
					    convertToCTL (tree.getRight (),
							  fairness));
    else if (data.equals (">="))
      return CTLFactory.createCTLOverNode (convertToCTL (tree.getLeft (),
							 fairness),
					   convertToCTL (tree.getRight (),
							 fairness));
    else if (data.equals ("EX"))
      return CTLFactory.createCTLEXNode (convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("AX"))
      return CTLFactory.createCTLAXNode (convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("EU"))
      return CTLFactory.createCTLEUNode (convertToCTL (tree.getLeft (),
						       fairness),
					 convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("AU"))
      return CTLFactory.createCTLAUNode (convertToCTL (tree.getLeft (),
						       fairness),
					 convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("EF"))
      return CTLFactory.createCTLEFNode (convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("AF"))
      return CTLFactory.createCTLAFNode (convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("EG"))
      return CTLFactory.createCTLEGNode (convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("AG"))
      return CTLFactory.createCTLAGNode (convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("AW"))
      return CTLFactory.createCTLAWNode (convertToCTL (tree.getLeft (),
						       fairness),
					 convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.equals ("EW"))
      return CTLFactory.createCTLEWNode (convertToCTL (tree.getLeft (),
						       fairness),
					 convertToCTL (tree.getRight (),
						       fairness),
					 fairness);
    else if (data.startsWith ("?"))
      {
	// get the name of the placeholder variable
	String phname = data.substring (1);
	List props = (List) tree.getRight ().getData ();
	String [] proplist = new String [props.size()];

	// get the list of relevant propostitions
	for (ListIterator i = props.listIterator (); i.hasNext (); )
	  {
	    proplist [i.nextIndex ()] =
	      ((BTNode) i.next ()).getData ().toString ();
	  }

	return CTLFactory.createCTLPlaceholderNode (phname, proplist,
						    fairness);
      }
    else if (lattice != null)
      {
	int value = lattice.getElementIdxByKey (data);
	if (value != Algebra.NO_VALUE)
	  return CTLFactory.createCTLLatticeNode (value, lattice);
      }
    // if it's not anything above, it's got to be an atom
    return CTLFactory.createCTLAtomNode (data, fairness);
  }

  public static void main (String[] args) throws Exception
  {
    BTNode bintree = null;
    CTLNode ctl;

    // -- prepare to read from stdin
    BufferedReader in = 
      new BufferedReader (new InputStreamReader (System.in));
    
    // -- read a ctl formula
    System.out.println ("Please enter a CTL property");
    String ctlStr = in.readLine ();

    // -- parse
    System.out.println ("Parsing: " + ctlStr);
    ctl = BinaryTreeToCTLConverter.convertToCTL (ctlStr);

    System.out.println ("The formula is " + ctl + ctl.getFairness ());
  }

  /**
   ** Check whether the string is a proper CTL formula.
   **/
  public static boolean isProperCTL (String str)
  {
    try {
      convertToCTL (str);
      return true;
    }
    catch (RuntimeException e) { return false; }
  }
}
