package edu.toronto.cs.ctl;

import java.util.*;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;

public class CTLFactory
{

  static Hashtable uniquetable = new Hashtable ();

  // no fairness
  public static CTLAndNode createCTLAndNode (CTLNode left, CTLNode right)
  {
    return createCTLAndNode (left, right, new LinkedList ());
  }

  public static CTLOrNode createCTLOrNode (CTLNode left, CTLNode right)
  {
    return createCTLOrNode (left, right, new LinkedList ());
  }

  public static CTLNegNode createCTLNegNode (CTLNode right)
  {
    return createCTLNegNode (right, new LinkedList ());
  }

  public static CTLImplNode createCTLImplNode (CTLNode left, CTLNode right)
  {
    return createCTLImplNode (left, right, new LinkedList ());
  }

  public static CTLIffNode createCTLIffNode (CTLNode left, CTLNode right)
  {
    return createCTLIffNode (left, right, new LinkedList ());
  }

  public static CTLEXNode createCTLEXNode (CTLNode right)
  {
    return createCTLEXNode (right, new LinkedList ());
  }

  public static CTLAXNode createCTLAXNode (CTLNode right)
  {
    return createCTLAXNode (right, new LinkedList ());
  }

  public static CTLPreEXNode createCTLPreEXNode (CTLNode right)
  {
    return createCTLPreEXNode (right, new LinkedList ());
  }

  public static CTLEFNode createCTLEFNode (CTLNode right)
  {
    return createCTLEFNode (right, new LinkedList ());
  }

  public static CTLAFNode createCTLAFNode (CTLNode right)
  {
    return createCTLAFNode (right, new LinkedList ());
  }

  public static CTLEUNode createCTLEUNode (CTLNode left, CTLNode right)
  {
    return createCTLEUNode (left, right, new LinkedList ());
  }

  public static CTLAUNode createCTLAUNode (CTLNode left, CTLNode right)
  {
    return createCTLAUNode (left, right, new LinkedList ());
  }

  public static CTLEUiNode createCTLEUiNode (CTLNode left, int i,
					     CTLNode right)
  {
    return createCTLEUiNode (left, i, right, new LinkedList ());
  }

  public static CTLAUiNode createCTLAUiNode (CTLNode left, int i,
					     CTLNode right)
  {
    return createCTLAUiNode (left, i, right, new LinkedList ());
  }

  public static CTLEGNode createCTLEGNode (CTLNode right)
  {
    return createCTLEGNode (right, new LinkedList ());
  }

  public static CTLAGNode createCTLAGNode (CTLNode right)
  {
    return createCTLAGNode (right, new LinkedList ());
  }

  public static CTLAtomNode createCTLAtomNode (Object data)
  {
    return createCTLAtomNode (data, new LinkedList ());
  }

  public static CTLLatticeNode createCTLLatticeNode (int value,
						     FiniteLattice l)
  {
    return createCTLLatticeNode (value, l, new LinkedList ());
  }
  
  public static CTLEqualsNode createCTLEqualsNode (CTLNode left,
						   CTLNode right)
  {
    return createCTLEqualsNode (left, right, new LinkedList ());
  }

  public static CTLUnderNode createCTLUnderNode (CTLNode left, CTLNode right)
  {
    return createCTLUnderNode (left, right, new LinkedList ());
  }

  public static CTLOverNode createCTLOverNode (CTLNode left, CTLNode right)
  {
    return createCTLOverNode (left, right, new LinkedList ());
  }

  public static CTLMvSetNode createCTLMvSetNode (MvSet mvset)
  {
    return createCTLMvSetNode (mvset, new LinkedList ());
  }

  public static CTLAWNode createCTLAWNode (CTLNode left, CTLNode right)
  {
    return createCTLAWNode (left, right, new LinkedList ());
  }

  public static CTLEWNode createCTLEWNode (CTLNode left, CTLNode right)
  {
    return createCTLEWNode (left, right, new LinkedList ());
  }

  public static CTLPlaceholderNode createCTLPlaceholderNode (Object name,
							     Object [] props)
  {
    return createCTLPlaceholderNode (name, props, new LinkedList ());
  }

  // fairness
  public static CTLAndNode createCTLAndNode (CTLNode left, CTLNode right,
					     List fairness)
  {
    CTLAbstractNode ctl = new CTLAndNode (left, right);
    ctl.setFairness (fairness);
    return (CTLAndNode) makeUnique (ctl);
  }

  public static CTLOrNode createCTLOrNode (CTLNode left, CTLNode right,
					   List fairness)
  {
    CTLAbstractNode ctl = new CTLOrNode (left, right);
    ctl.setFairness (fairness);
    return (CTLOrNode) makeUnique (ctl);
  }

  public static CTLNegNode createCTLNegNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLNegNode (right);
    ctl.setFairness (fairness);
    return (CTLNegNode) makeUnique (ctl);
  }

  public static CTLImplNode createCTLImplNode (CTLNode left, CTLNode right,
					       List fairness)
  {
    CTLAbstractNode ctl = new CTLImplNode (left, right);
    ctl.setFairness (fairness);
    return (CTLImplNode) makeUnique (ctl);
  }

  public static CTLIffNode createCTLIffNode (CTLNode left, CTLNode right,
					     List fairness)
  {
    CTLAbstractNode ctl = new CTLIffNode (left, right);
    ctl.setFairness (fairness);
    return (CTLIffNode) makeUnique (ctl);
  }

  public static CTLEXNode createCTLEXNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLEXNode (right);
    ctl.setFairness (fairness);
    return (CTLEXNode) makeUnique (ctl);
  }

  public static CTLAXNode createCTLAXNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLAXNode (right);
    ctl.setFairness (fairness);
    return (CTLAXNode) makeUnique (ctl);
  }

  public static CTLPreEXNode createCTLPreEXNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLPreEXNode (right);
    ctl.setFairness (fairness);
    return (CTLPreEXNode) makeUnique (ctl);
  }

  public static CTLEFNode createCTLEFNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLEFNode (right);
    ctl.setFairness (fairness);
    return (CTLEFNode) makeUnique (ctl);
  }

  public static CTLAFNode createCTLAFNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLAFNode (right);
    ctl.setFairness (fairness);
    return (CTLAFNode) makeUnique (ctl);
  }

  public static CTLEUNode createCTLEUNode (CTLNode left, CTLNode right,
					   List fairness)
  {
    CTLAbstractNode ctl = new CTLEUNode (left, right);
    ctl.setFairness (fairness);
    return (CTLEUNode) makeUnique (ctl);
  }

  public static CTLAUNode createCTLAUNode (CTLNode left, CTLNode right,
					   List fairness)
  {
    CTLAbstractNode ctl = new CTLAUNode (left, right);
    ctl.setFairness (fairness);
    return (CTLAUNode) makeUnique (ctl);
  }

  public static CTLEUiNode createCTLEUiNode (CTLNode left, int i,
					     CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLEUiNode (left, i, right);
    ctl.setFairness (fairness);
    return (CTLEUiNode) makeUnique (ctl);
  }

  public static CTLAUiNode createCTLAUiNode (CTLNode left, int i,
					     CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLAUiNode (left, i, right);
    ctl.setFairness (fairness);
    return (CTLAUiNode) makeUnique (ctl);
  }

  public static CTLEGNode createCTLEGNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLEGNode (right);
    ctl.setFairness (fairness);
    return (CTLEGNode) makeUnique (ctl);
  }

  public static CTLAGNode createCTLAGNode (CTLNode right, List fairness)
  {
    CTLAbstractNode ctl = new CTLAGNode (right);
    ctl.setFairness (fairness);
    return (CTLAGNode) makeUnique (ctl);
  }

  public static CTLAtomNode createCTLAtomNode (Object data, List fairness)
  {
    CTLAbstractNode ctl = new CTLAtomNode (data);
    ctl.setFairness (fairness);
    return (CTLAtomNode) makeUnique (ctl);
  }

  public static CTLLatticeNode createCTLLatticeNode (int value,
						     FiniteLattice l,
						     List fairness)
  {
    CTLAbstractNode ctl = new CTLLatticeNode (value, l);
    ctl.setFairness (fairness);
    return (CTLLatticeNode) makeUnique (ctl);
  }
  
  public static CTLEqualsNode createCTLEqualsNode (CTLNode left,
						   CTLNode right,
						   List fairness)
  {
    CTLAbstractNode ctl = new CTLEqualsNode (left, right);
    ctl.setFairness (fairness);
    return (CTLEqualsNode) makeUnique (ctl);
  }

  public static CTLUnderNode createCTLUnderNode (CTLNode left, CTLNode right,
						 List fairness)
  {
    CTLAbstractNode ctl = new CTLUnderNode (left, right);
    ctl.setFairness (fairness);
    return (CTLUnderNode) makeUnique (ctl);
  }

  public static CTLOverNode createCTLOverNode (CTLNode left, CTLNode right,
					       List fairness)
  {
    CTLAbstractNode ctl = new CTLOverNode (left, right);
    ctl.setFairness (fairness);
    return (CTLOverNode) makeUnique (ctl);
  }

  public static CTLMvSetNode createCTLMvSetNode (MvSet mvset, List fairness)
  {
    CTLAbstractNode ctl = new CTLMvSetNode (mvset);
    ctl.setFairness (fairness);
    return (CTLMvSetNode) makeUnique (ctl);
  }

  public static CTLAWNode createCTLAWNode (CTLNode left, CTLNode right,
					   List fairness)
  {
    CTLAbstractNode ctl = new CTLAWNode (left, right);
    ctl.setFairness (fairness);
    return (CTLAWNode) makeUnique (ctl);
  }

  public static CTLEWNode createCTLEWNode (CTLNode left, CTLNode right,
					   List fairness)
  {
    CTLAbstractNode ctl = new CTLEWNode (left, right);
    ctl.setFairness (fairness);
    return (CTLEWNode) makeUnique (ctl);
  }

  public static CTLPlaceholderNode createCTLPlaceholderNode (Object name,
							     Object [] props,
							     List fairness)
  {
    CTLAbstractNode ctl = new CTLPlaceholderNode (name, props);
    ctl.setFairness (fairness);
    return (CTLPlaceholderNode) makeUnique (ctl);
  }

  public static void removeCTLNode (CTLNode ctl)
  {
    uniquetable.remove (ctl.toString ());
  }

  // some private methods

  /**
   ** Make sure that a CTLNode is fair.  Fairness List c is
   ** ignored if the node is anything other than a leaf node.
   **
  private static CTLNode makeFair (CTLNode ctl, List c)
  {
    // check that it is not a leaf node
    if (!(ctl.getLeft () == null) || !(ctl.getRight () == null))
      {
	// make sure the node has the same fairiness as its children
	List c1 = null, c2 = null;
	// left child may not exist    
	c1 = ctl.getRight ().getFairness ();
	if (ctl.getLeft () != null)
	  {
	    c2 = ctl.getLeft ().getFairness ();
	  }
	else // just make them the same
	  {
	    c2 = c1;
	  }
	if (c1.equals (c2))
	  {
	    c.addAll (c1);
	    ((CTLAbstractNode) ctl).setFairness (c);
	  }
	else throw new IllegalArgumentException
	       ("CTLNode's - " + ctl +
		" - children must have the same fairness.");
      }
    ((CTLAbstractNode) ctl).setFairness (c);
    
    return ctl;
  }
  private static CTLNode makeFair (CTLNode ctl)
  {
    return makeFair (ctl, new LinkedList ());
  }
  */
  /**
   ** Make sure that a CTLNode is unique.
   **/
  private static CTLNode makeUnique (CTLNode ctl)
  {
    String key = ctl.toFairString ();

    if (uniquetable.containsKey (key)) // we already have it
      {
	return (CTLNode) uniquetable.get (key);
      }
    else // we don't have it
      {
	uniquetable.put (key, ctl);
	return ctl;
      }
  }
   
  /**
   ** Creates a CTL node that is the same type as the original and
   ** has the specified children.
   **/
  public static CTLNode alterChildren (CTLNode psi, CTLNode left,
				       CTLNode right)
  {
    // make a copy of a node
    CTLAbstractNode phi = makeSameNode (psi);
    
    // change the children
    phi.setLeft (left);
    phi.setRight (right);
    return makeUnique (phi);
  }

  /**
   ** HACK!!!!!!
   **/
  public static CTLNode makeSameAndFair (CTLNode psi, List c)
  {
    CTLAbstractNode phi = makeSameNode (psi);
    phi.setFairness (c);
    return makeUnique (phi);
  }
  
  /**
   ** This is essentially same as cloning, but it is not public, and
   ** only works for certain nodes.
   **/
  private static CTLAbstractNode makeSameNode (CTLNode psi)
  {
    CTLAbstractNode result;

    // we only ever need to deal with non-leaf nodes
    if (psi instanceof CTLNegNode)
      result = new CTLNegNode (psi.getRight ());
    else if (psi instanceof CTLAndNode)
      result = new CTLAndNode (psi.getLeft (), psi.getRight ());
    else if (psi instanceof CTLOrNode)
      result = new CTLOrNode (psi.getLeft (), psi.getRight ());
    else if (psi instanceof CTLAXNode)
      result = new CTLAXNode (psi.getRight ());
    else if (psi instanceof CTLAUNode)
      result = new CTLAUNode (psi.getLeft (), psi.getRight ());
    else if (psi instanceof CTLAGNode)
      result = new CTLAGNode (psi.getRight ());
    else if (psi instanceof CTLEXNode)
      result = new CTLEXNode (psi.getRight ());
    else if (psi instanceof CTLEUNode)
      result = new CTLEUNode (psi.getLeft (), psi.getRight ());
    else if (psi instanceof CTLEGNode)
      result = new CTLEGNode (psi.getRight ());
    else if (psi instanceof CTLEqualsNode)
      result = new CTLEqualsNode (psi.getLeft (), psi.getRight ());
    else if (psi instanceof CTLOverNode)
      result = new CTLOverNode (psi.getLeft (), psi.getRight ());
    else if (psi instanceof CTLUnderNode)
      result = new CTLUnderNode (psi.getLeft (), psi.getRight ());
    // apparently it's a leaf node
    else result = (CTLAbstractNode) psi;

    result.setFairness (psi.getFairness ());
    return result;
  }
 
}

