package edu.toronto.cs.ctl;

import java.util.*;

/**
 ** This is an abstract CTL node.
 **/
public interface CTLNode
{

  /**
   ** Retrieves the left child of the CTLNode.
   **
   ** @return left child.
   **/
  CTLNode getLeft ();

  /**
   ** Retrieves the right child of the CTLNode.
   **
   ** @return right child.
   **/
  CTLNode getRight ();

  /**
   ** Retrieves the fairness of the CTLNode.
   **
   ** @return fairness for this CTL.
   **/
  List getFairness ();
    
  /**
   ** CTL operation - conjunction.
   **/
  CTLAndNode and (CTLNode psi);

  /**
   ** CTL operation - disjunction.
   **/
  CTLOrNode or (CTLNode psi);

  /**
   ** CTL operation - negation.
   **/
  CTLNegNode neg ();

  /**
   ** CTL operation - implication.
   **/
  CTLImplNode implies (CTLNode psi);

  /**
   ** CTL operation - backward implication.
   **/
  CTLImplNode impliedBy (CTLNode psi);

  /**
   ** CTL operation - equivalence.
   **/
  CTLIffNode iff (CTLNode psi);

  /**
   ** CTL operation - EX.
   **/
  CTLEXNode ex ();

  /**
   ** CTL operation - AX.
   **/
  CTLAXNode ax ();

  /**
   ** CTL operation - EF.
   **/
  CTLEFNode ef ();

  /**
   ** CTL operation - AF.
   **/
  CTLAFNode af ();

  /**
   ** CTL operation - EU ( E[this U psi] ).
   **/
  CTLEUNode eu (CTLNode psi);

  /**
   ** CTL operation - bounded EU ( E[this Ui psi] ).
   **/
  CTLEUiNode eu (int i, CTLNode psi);

  /**
   ** CTL operation - AU ( A[this U psi] ).
   **/
  CTLAUNode au (CTLNode psi);

  /**
   ** CTL operation - bounded AU ( A[this Ui psi] ).
   **/
  CTLAUiNode au (int i, CTLNode psi);

  public void accept (CTLVisitor v, Object stateinfo);
  

  /**
   ** CTL operation - EG.
   **/
  CTLEGNode eg ();

  /**
   ** CTL operation - AG.
   **/
  CTLAGNode ag ();

  /**
   ** CTL operation - equals
   **/
  CTLEqualsNode eq (CTLNode psi);

  /**
   ** CTL operation - under
   **/
  CTLUnderNode under (CTLNode psi);

  /**
   ** CTL operation - over
   **/
  CTLOverNode over (CTLNode psi);

  /**
   ** This is used to determine uniqueness.
   **/
  String toFairString ();
}


