package edu.toronto.cs.experiments;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.util.*;

import edu.toronto.cs.mvset.*;

public class BddEmbedExperiment
{

  // -- number of cubes to alter
  FiniteLattice lattice;

  int top;
  int bot;


  ExperimentOutput output;
  

  String ddFileName;
  
  public BddEmbedExperiment (FiniteLattice _lattice, 
			      ExperimentOutput _output, 
			      String _ddFileName)
  {
    
    lattice = _lattice;
    output = _output;
    ddFileName = _ddFileName;

    top = lattice.idtimesIdx ();
    bot = lattice.idplusIdx ();
    
    
    
  }
  

  public void runExperiment ()
  {

    // -- create our mv-set factory and set some arrity...
    MvSetFactory mvSetFactory = CUBDDMvSet.newMvSetFactory (lattice, 30);
    
    

    
    // -- read in the bdd and transfer it into our factory
    MvSet bdd = DDUnDumper.readBDDSafe (mvSetFactory, ddFileName);

    // -- first sample is ready
    output.addSample ("bdd", bdd.size ());
    
    // -- reorder the diagram
    bdd.reorder ();
    // -- second sample is ready
    output.addSample ("reorder_bdd", bdd.size ());
    output.flush ();

    // -- done
  }
  

			   
  

  public static void main (String[] args) throws Exception
  {
    // -- intialize seed so that we could re-run this
    Chaos.seed (0);
    
    String ddFileName = args [0];
    String outputDir = args [1];
    String experimentName = args [2];


    String latticeName = args [3];
    
    
    ExperimentOutput output = 
      new ExperimentOutput (outputDir, experimentName);
    FiniteLattice lattice;

    
    
    lattice = Lattices.getLatticeByName (latticeName);
    System.out.println ("Getting a lattice: " + latticeName);

    BddEmbedExperiment experiment = 
      new BddEmbedExperiment (lattice, 
			      output, 
			      ddFileName);
    
    System.out.println ("Running experiment.... on " + ddFileName);
    experiment.runExperiment ();
    System.out.println ("Done");
  }
  
}
