package edu.toronto.cs.experiments;

import edu.toronto.cs.mvset.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.util.*;


import java.util.*;


public class ModelCheckingFunction extends MvFunction
{
  int supportLow;
  int supportHigh;
  
  // -- size in number of minterms or Alices or what we call them
  int size;
  Set entries;

  
  public ModelCheckingFunction (FiniteLattice lattice, 
				int arrity, int _supportLow, 
				int _supportHigh, int _size)
  {
    super (lattice, arrity);
    supportLow = _supportLow;
    supportHigh = _supportHigh;
    size = _size;
    entries = new HashSet ();

    
    
    // -- generate the function
    generate ();
  }
  

  void generate ()
  {
    // -- add size unique entries
    for (int i = 0; i < size; i++)
      {
	//System.out.println ("Adding entry: " + i);
	
	while (!addUniqueEntry (getMinterm (), getValue ()));
      }
    
    
      //while (!addUniqueEntry (getMinterm (), getValue ()));
    
  }

  int[] getMinterm ()
  {
    int[] minterm = new int [arrity];
    Arrays.fill (minterm, Algebra.NO_VALUE);

    
    // -- variables seen so far
    boolean[] seenVars = new boolean [arrity];
    Arrays.fill (seenVars, false);

    // -- number of vars seen so far
    int seenVarsCount = 0;
    

    // -- pick number of variables in which this thing depends
    int supportSize = Chaos.inRange (supportLow, supportHigh);
    while (seenVarsCount < supportSize)
      {
	// -- pick a variable
	int var = Chaos.nextUpTo (arrity);
	// -- skip if we done this variable already
	if (seenVars [var]) continue;

	// -- mark as seen
	seenVars [var] = true;
	// -- add this to our minterm
	minterm [var] = getVarValue ();
	seenVarsCount++;
      }
    return minterm;
  }
  
  // -- this can be overriden to support a different policy for values
  int getVarValue ()
  {
    //return Chaos.nextUpTo (lattice.size ());
    if (Chaos.nextThreshold (0.5f))
      return lattice.idtimesIdx ();
    else
      return lattice.idplusIdx ();
  }
  
  // -- returns a non-false value
  int getValue ()
  {
    return Chaos.inRange (0, lattice.size () - 1);
  }
  
  boolean addUniqueEntry (int[] minterm, int value)
  {
    MvEntry entry = new MvEntry (minterm, value);
    // -- if we already have this entry, skip it
    if (entries.contains (entry)) return false;
    // -- this is a new entry, so add it
    entries.add (entry);
    return true;
  }
  

  public String toString ()
  {
    StringBuffer sb = new StringBuffer ();

    for (Iterator it = entries.iterator (); it.hasNext ();)
      {
	MvEntry entry = (MvEntry)it.next ();
	sb.append (mintermToString (entry.getMinterm ()));
	sb.append (' ');
	sb.append (valueToString (entry.getValue ()));
	sb.append ('\n');
      }
    return sb.toString ();
  }
  
  public MvSet _toMvSet (MvSetFactory factory)
  {
    MvSet result = factory.createConstant (bot);
    MvSet subResult = null;
    MvSet topMvSet = factory.createConstant (lattice.idtimesIdx ());
    
    int counter = 0;
    
    for (Iterator it = entries.iterator (); it.hasNext ();)
      {
	if (counter % 3 == 0)
	  {
	    if (subResult != null)
	      result = result.or (subResult);
	    subResult = topMvSet;
	  }
	else
	  {
	    subResult = 
	      subResult.and (((MvEntry)it.next ()).toMvSet (factory));
	  }
	counter++;
	//System.out.println ("Finished doing entry: " + counter);
      }

    if (subResult != null && subResult != topMvSet)
      result = result.or (subResult);
    return result;
  }
  
  public MvSet toMvSet (MvSetFactory factory)
  {
    /*** Assume that each entry is a valid 'minterm', so 
     *** just OR them all together
     ***/
    MvSet result = factory.createConstant (bot);
   
    int count = 0;
    for (Iterator it = entries.iterator (); it.hasNext ();)
      {
	//System.out.println ("DD Adding entry: " + (count++));
	
	result = result.or (((MvEntry)it.next ()).toMvSet (factory));
      }
    
    
    return result;
  }

  // -- returns a new function of similar parameters
  public MvFunction newMvFunction ()
  {
    return new ModelCheckingFunction (lattice, arrity, supportLow, 
				      supportHigh, size);
  }
  



  public static void main (String[] args)
  {
    for (Iterator it = Lattices.availableLattices ().iterator ();
	 it.hasNext ();)
      {
	String latticeName = (String)it.next ();
	FiniteLattice lattice = Lattices.getLatticeByName (latticeName);
	
	for (int i = 10; i <= 60; i += 10)
	  {
	    System.out.println ("Doing: " + latticeName + " " + i);
	    MvFunction function = null;
	    for (int j = 0; j < 10; j++)
	      {
	    MvSetFactory factory = 
	      CUBDDMvSet.newMvSetFactory (lattice, 10);

		System.out.println ("Doing function: " + j);
		if (function == null)
		  function = new ModelCheckingFunction (lattice, 10, 3, 5, i);
		else
		  function = function.newMvFunction ();
		
//  		System.out.println ("Converting");
//  		MvSet mvset = function.toMvSet (factory);
// 		System.out.println ("Size: " + mvset.size ());
// 		mvset.reorder ();
// 		System.out.println ("Size after reordering: "  + 
// 				    mvset.size ());
// 		factory = null;
// 		mvset = null;
// 		System.gc ();
	      }
// 	    System.gc ();
	  }
	
      }
    
  }
  
  public static void __main (String[] args)
  {
    FiniteLattice lattice = Lattices.getLatticeByName ("2Bool");

    
    
    System.out.println ("Generating a function");
    MvFunction function = new ModelCheckingFunction (lattice, 10, 3, 5, 20);
    
    
    for (int i = 0; i < 10; i++)
      {
	MvSetFactory factory = MDDMvSet.newMvSetFactory (lattice, 10);
	MvSetFactory factory2 = JMvSet.newMvSetFactory (lattice, 10);
	MvSetFactory factory3 = CUADDMvSet.newMvSetFactory (lattice, 10);
	MvSetFactory factory4 = CUBDDMvSet.newMvSetFactory (lattice, 10);

	function = function.newMvFunction ();	
	System.out.println ("Generating DD");
	MvSet mvSet = function.toMvSet (factory);
	System.out.println ("The size is: " + mvSet.size ());
	mvSet = function.toMvSet (factory2);
	System.out.println ("The mtbdd size is: " + mvSet.size ());
	mvSet = function.toMvSet (factory3);
	System.out.println ("CUADD size before reorder: " + mvSet.size ());
	mvSet.reorder ();
	System.out.println ("The CUADD size is: " + mvSet.size ());
	mvSet = function.toMvSet (factory4);
	System.out.println ("CUBDD size before reorder: " + mvSet.size ());
	mvSet.reorder ();
	System.out.println ("The CUBDD size is: " + mvSet.size ());
	
	mvSet = null;
	factory = null;
	factory2 = null;
	factory3 = null;
	factory4 = null;
	//System.gc ();
      }
    
    
  }
  
  
}
