package edu.toronto.cs.experiments;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.util.*;


public class MvProductExperiment
{

  // -- number of cubes to alter
  int noiseLevel;
  FiniteLattice lattice;

  int top;
  int bot;

  ChaosIterator varDomain;

  ExperimentOutput output;
  
  // -- number of viewpoints we are merging
  int viewpoints;

  String ddFileName;
  
  public MvProductExperiment (FiniteLattice _lattice, 
			      ExperimentOutput _output, 
			      int _viewpoints,
			      String _ddFileName,
			      int _noiseLevel)
  {
    
    lattice = _lattice;
    output = _output;
    viewpoints = _viewpoints;
    ddFileName = _ddFileName;
    noiseLevel = _noiseLevel;

    top = lattice.idtimesIdx ();
    bot = lattice.idplusIdx ();
    
    varDomain = new UniformChaosIterator (new int[] {top, bot});
    
    
  }
  

  public void runExperiment ()
  {

    // -- create our manager XXX ignore arrity
    MDDManager manager = new MDDMgr (0, lattice);
    
    MDDNode[] nodes = new MDDNode [viewpoints];
    
    // -- first view point comes from a file
    nodes [0] = DDUnDumper.readBDDSafe (manager, ddFileName);

    //System.out.println ("Original size: " + manager.dagSize (nodes [0]));

    // -- the rest are obtained from mv-noise
    for (int i = 1; i < nodes.length; i++)
      nodes [i] = introduceMvNoise (manager, nodes [0]);

//     System.out.println ("Did we do anything: " + 
// 			(nodes [0] != nodes [1]));
    
    // -- we are ready for the 'mbtdd' case
    output.addSample ("mbtdd", manager.sharedSize (nodes));
    

    // -- now we need to create the product
    MDDNode product = nodes [0];
    for (int i = 1; i < nodes.length; i++)
      product = manager.apply (product, nodes [i], 
			       new MDDMvProductFunction (lattice, i));

    // -- finally we are ready with the second answer
    output.addSample ("mdd", manager.dagSize (product));

    output.flush ();
  }
  

			      

  // -- introduce mv-noise into a decision diagram
  public  MDDNode introduceMvNoise (MDDManager manager, MDDNode node)
  {
    MDDNode positiveCubes = manager.getZero ();
    MDDNode negativeCubes = manager.getZero ();

    // -- find cubes we will change
    for (int i = 0; i < noiseLevel; i++)
      {
// 	System.out.println ("Looking for a cube: " + i);
	// -- get a random poin
	MDDPoint point = manager.getRandomPoint (node, varDomain);
	if (point.getValue () == top)
	  // -- got a top cube
	  positiveCubes = manager.Apply (positiveCubes, point.getNode (), 
					 FiniteLattice.JOIN);
	else 
	  negativeCubes = manager.Apply (negativeCubes, point.getNode (),
					 FiniteLattice.JOIN);
      }
    



    MDDNode result = node;

    System.out.println ("Merging all negative cubes...");
    // -- change all of the negative cubes    
    result = manager.Apply (result, negativeCubes, 
			    FiniteLattice.JOIN);
    
    System.out.println ("Merging all positive cubes....");
    // -- change all positive cubes
    result = manager.Apply 
      (result, manager.Apply (positiveCubes, FiniteLattice.NEG),
       FiniteLattice.MEET);
    return result;
  }
  

  public static void main (String[] args) throws Exception
  {
    // -- intialize seed so that we could re-run this
    Chaos.seed (0);
    
    String ddFileName = args [0];
    String outputDir = args [1];
    String experimentName = args [2];


    int viewpoints = Integer.parseInt (args [3]);
    int noiseLevel = Integer.parseInt (args [4]);
    
    
    ExperimentOutput output = 
      new ExperimentOutput (outputDir, experimentName);
    FiniteLattice lattice;

    
    int latticeSize = 1 << viewpoints;
    
    lattice = Lattices.getLatticeByName (latticeSize + "Bool");
    System.out.println ("Getting a lattice: " + latticeSize + "Bool");

    for (int i = 0; i < 100; i++)
      {
	System.out.println ("Getting sample: " + i);
	MvProductExperiment experiment = new MvProductExperiment (lattice, 
								  output, 
								  viewpoints, 
								  ddFileName,
								  noiseLevel);
	System.out.println ("Running experiment....");
	experiment.runExperiment ();
	System.out.println ("Done");
      }
    
  }
  
}
