package edu.toronto.cs.gui;

import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.tlq.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.util.dgraph.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

import javax.swing.table.*;

/**
 * LatticeTableModel.java
 *
 *
 * Created: Tue Jun 25 16:53:55 2002
 *
 * @author <a href="mailto:anya@tallinn.cs">Tafliovich Anya</a>
 * @version
 */

public class LatticeTableModel extends AbstractTableModel {
 
  // a lattice for which the table is displayed
  FiniteLattice lattice;
  
  // the type of the table - meet, join, or negation,
  // as defined in FiniteLattice
  int type;
  
  public LatticeTableModel (FiniteLattice l, int t){
    super();
    lattice = l;
    type = t;
  }

  // the number of columns is the size of the lattice plus 1  
  public int getColumnCount () 
  {
    return lattice.size()+1;
  }

  // if this is the model for a negation table,
  // then there is only 1 row; otherwise, the number
  // of rows is the same as the number of columns
  public int getRowCount ()
  {
    if (type == FiniteLattice.NEG) return 1;
    else return getColumnCount() - 1;
  }
  

  // the values in the first column duplicate the header values
  // (unless this is a negation model);
  // the rest of the values are determined by an operation corresponding
  // to the type of this model
  public Object getValueAt (int row, int col)
  {
    
    if (type == FiniteLattice.NEG)
      {
	if (col == 0) 
	  return null;
	else
	  return lattice.getElementKeyByIdx (lattice.getNegTable()[col-1]);
      }
    else // meet or join
      {
	if (col == 0)
	  // duplicate the header values
	  return lattice.getElementKeyByIdx (row);
	
	else
	  {
	    if (type == FiniteLattice.MEET)
	      return lattice.getElementKeyByIdx 
                      (lattice.getMeetTable()[row][col-1]);
	    else
	      return lattice.getElementKeyByIdx 
		(lattice.getJoinTable()[row][col-1]);
	  }
      }
  }

  // the names of the columns are the values of the lattice in order
  public String getColumnName (int col)
  {
    if (col == 0) return null;
    else return (String) lattice.getElementKeyByIdx (col-1);
  }
  
}// LatticeTableModel




