/*
 * KVWState.java
 *
 * Created on November 22, 2001, 12:03 PM
 */

package edu.toronto.cs.kegvis;
import java.util.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.modelchecker.PropOrderIf;
import edu.toronto.cs.util.Assert;
import edu.toronto.cs.algebra.Algebra;
/**
 *
 * @author  bwong
 * @version
 */
public class KVWState implements KVState {
  
  String label;
  
  Vector prop;
  
  Vector ctl;
  
  HashSet flags;
  
  KEGStateNode statenode;
  
  PropOrderIf propOrdering;
  
  /** a list of neighbouring nodes */
  Vector neighbours;
  
  /** a value for the transition from the previous state */
  String transValue;
  
  /** Creates new KVWState */
  public KVWState(KEGStateNode statenode, PropOrderIf propOrdering, Vector neighbours, Object transValueObj) {
    this(statenode, propOrdering);
    this.neighbours = neighbours;
    this.transValue = transValueObj.toString();
  }
  public KVWState(KEGStateNode statenode, PropOrderIf propOrdering) {
    this.statenode = statenode;
    this.propOrdering = propOrdering;
    
    // set up the properties
    prop = setupProperties();
    ctl = new Vector();
    
    // set up a CTL node in this graph
    if (statenode.getCTLGraph() != null) {
      try {
        // TODO: does not support 2 CTL paths converging on another state
        //       (two different EX formulas explained by the same state)
        KEGCtlNode firstctl =
        (KEGCtlNode) statenode.getCTLGraph().getRootNodes().iterator().next();
        ctl.add(new KVWCtlExpression(firstctl));
      }
      catch (NoSuchElementException e) {
        Assert.assert(e);
      }
      
    }
  }
  
  public boolean isLabelized() {
    return (label != null);
  }
  
  /** Getter for property label.
   * @return Value of property label.
   */
  public java.lang.String getLabel() {
    return label;
  }
  
  /** Getter for property ctl.
   * @return Value of property ctl.
   */
  public java.util.Vector getCtl() {
    return ctl;
  }
  
  public Iterator getCtlIterator() {
    return ctl.iterator();
  }
  
  public Iterator getPropIterator() {
    return prop.iterator();
  }
  
  /** This method generates the state properties Vector prop.
   * It is a modified version of MvState.toString(PropOrderIf props)
   * Some work may be needed in MvState to provide better human interface
   * @return Vector of KVWStateProperty in this state.
   */
  Vector setupProperties() {
    Vector prop = new Vector();
    Algebra lattice = statenode.getState().getLattice();
    int [] upp = propOrdering.getUnPrimedProps();
    int [] restrictions = statenode.getState().getRestrictions();
    for (int i=0; i < upp.length; i++) {
      KVWStateProperty newprop = new KVWStateProperty();
      if (propOrdering.getPropName(upp[i]).charAt(0) == '_') {
        if (restrictions[upp[i]] == lattice.idtimesIdx()) {
          newprop.setName(propOrdering.getPropName(upp[i]));
          if (restrictions[upp[i]] != Algebra.NO_VALUE)
            newprop.setValue((String)lattice.getElementKeyByIdx(restrictions [upp[i]]));
          else
            newprop.setValue("NO_VAL");
        }
      }
      else {
        newprop.setName(propOrdering.getPropName(upp[i]));
        if (restrictions[upp[i]] != Algebra.NO_VALUE)
          newprop.setValue((String)lattice.getElementKeyByIdx(restrictions [upp[i]]));
        else
          newprop.setValue("NO_VAL");
      }
      prop.add(newprop);
    }
    return prop;
  }
  
  public int numSuccessors() {
    return statenode.getSuccessors().size();
  }
  
  public Vector getSuccessors() {
    return new Vector(statenode.getSuccessors());
  }

  /** gets the value of the transition to the given successor
   * @param successor The KEGStateNode to consider the edge to.
   * @return Integer value of the transition FROM this node TO given successor
   */
  public int getTransValue(KEGStateNode successor) {
    return ((KEGStateTransition) statenode.getEdge(successor)).getValue();
  }
  
  /** Getter for property transValue, the string representing the value of the
   * transition TO this node.
   * @return Value of property transValue.
   */
  public java.lang.String getTransValue() {
    return transValue;
  }
  
  /** Getter for property neighbours.
   * @return Value of property neighbours.
   */
  public java.util.Vector getNeighbours() {
    return neighbours;
  }
  
}
