package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;

public class Button extends SMVModule
{
  static final int PRESSED = 0;
  static final int REQUEST = 1;

  MvSet reset = null;
  
  public Button (String _name)
  {
    super (_name);
    vars = new String[]{"pressed", "request"};
  }
  

  public void setReset (MvSet v)
  {
    reset = v;
  }
  
  
  public MvSet computeTrans ()
  {
    System.out.println ("Computing trans for: " + getName ());
    return computeRequest ().and (computePressed ());
  }
  

  /*** Abstraction over a button using 3-val logic */
  public MvSet __computeTrans ()
  {
    int maybe = lattice.getElementIdxByKey ("M");
    
    /***
	next (request) := case
	                 reset: false;
			 request: request;
			 1: {true, false} : maybe transition
			 esac;
       The default clause corresponds to !reset & !request
    ***/
    MvSet pressed = cur(PRESSED, maybe).and (next (PRESSED, maybe));
    
    return new CaseStatement ().
      addCase (reset, next (REQUEST, bot)).
      addCase (cur (REQUEST), next (REQUEST)).
      addDefault (next (REQUEST, top, maybe).or
		  (next (REQUEST, bot, maybe))).compute ().and (pressed);
      
  }
  
  

  public MvSet computePressed ()
  {
    // -- pressed := {true, false}
    return next (PRESSED, top).or (next (PRESSED, bot));
  }
  
  public MvSet computeRequest ()
  {
    /***
     * request := case
     *             pressed & !reset: true;
     *             request & !reset: true;
     *      	   1               : false;
     *		  esac
     */
//      MvSet notReset = reset.not ();

//      return new CaseStatement ().
//        addCase (cur (PRESSED, top).and (notReset), next (REQUEST, top)).
//        addCase (cur (REQUEST, top).and (notReset), next (REQUEST, top)).
//        addDefault (next (REQUEST, bot)).compute ();

    MvSet requestIsRequest = 
      ddManager.createProjection (getVarOrder (REQUEST)).eq
      (ddManager.createProjection (getNextVarOrder (REQUEST)));
    /**
     * request := case
     *             request & reset : false;
                   pressed & !reset: true;
		   1 : request
    */

    return new CaseStatement ().
      addCase (cur (REQUEST).and (reset), next (REQUEST, bot)).
      addCase (cur (PRESSED).and (reset.not ()), next (REQUEST)).
      addDefault (requestIsRequest).compute ();
  }

  

  public MvState initializeInitState (MvState initState)
  {
    //initState.setValue (getVarOrder (PRESSED), bot);
    initState.setValue (getVarOrder (PRESSED), 
			lattice.getElementIdxByKey ("M"));
    initState.setValue (getVarOrder (REQUEST), bot);
    return initState;
  }
  
    
}
