package edu.toronto.cs.lift;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.algebra.*;

import edu.toronto.cs.util.*;

import java.util.Arrays;

public class MainLiftRAbst extends MainLiftRAbstract
{
  MvSetFactory dstFactory;

  public MainLiftRAbst (String name)
  {
    super (name);
  }
  
  public MainLiftRAbst (String name, int floors)
  {
    super (name, floors);
  }
  
  public void setMvSetFactory (MvSetFactory factory)
  {
    dstFactory = factory;
    MvSetFactory mFactory = MDDMvSet.newMvSetFactory (getLattice (), 
						      ordering.size ());
    super.setMvSetFactory (mFactory);
  }
  
    
  
  public MvSet computeTrans ()
  {
    MvSet result = computeButtons ().and (computeLift ());

    MDDManager srcManager = 
      ((MDDMvSet.MDDMvSetFactory)result.getFactory ()).getMddManager ();


    MDDManager dstManager = 
      ((MDDMvSet.MDDMvSetFactory)dstFactory).getMddManager ();
    
    
    MDDNode resultNode = ((MDDMvSet)result).getMddNode ();
    
    int[] logicMap = new int []{0, 1, 2};
    
    System.out.println ("Transfering to a clean MDDManager");
    resultNode = srcManager.transfer (dstManager, resultNode, logicMap);
    
    return ((MDDMvSet.MDDMvSetFactory)dstFactory).createMvSet (resultNode);
  }

  // -- Module intialization routine to be used by the model checker
  public static SMVModule initModule (String[] args)
  {
    System.out.println ("Intializing: " + MainLiftRAbst.class.getName ());
    System.out.println ("Will refer to this module as \'main\'");
    String moduleName = "main";
    System.out.println ("Expecting one argument: numer of floors");

    int floors;
    try {
      floors = Integer.parseInt (args [0]);
    } catch (Exception ex) {
      System.out.println ("Hmm no arguments, got: " + ex);
      System.out.println ("Assuming 3 floors");
      floors = 3;
    }
    
    
    String latticeFile = "play/gxl/3val-logic.xml";
    FiniteLattice lattice = null;
    System.out.println ("Using " + latticeFile + " as our lattice");
    try {
      lattice = LatticeFactory.parse (latticeFile);
    } catch (Exception ex) {
      System.out.println ("Failed to parse: " + latticeFile);
      ex.printStackTrace ();
      Assert.assert ("Failed to parse the lattice", ex);
    }
    
    MainLiftRAbst main = new MainLiftRAbst (moduleName, floors);
    main.setLattice (lattice);
    return main;
  }
  
  
}
