package edu.toronto.cs.mdd;

import java.util.Hashtable;

import edu.toronto.cs.util.Logger;

import edu.toronto.cs.util.StopWatch;

public class MDDCacheStub implements MDDNodeCache 
{

  //static Logger log = new Logger (MDDCacheStub.class.getName ());

  // XXX This assumes that no two threads call us at the same time
  MDDCacheKey lookupKey;
  
  Hashtable cache;

  StopWatch watch;
  StopWatch insertWatch;
 
  /** Constructor. */
  public MDDCacheStub() 
  {
    //cache = new Hashtable();
    cache = new Hashtable ();
    lookupKey = new MDDCacheKey ();
    watch = new StopWatch ();
    //watch.pause ();
    insertWatch = new StopWatch ();
    //watch.pause ();
  }

  public MDDNode find (MDDNode m1, int oper, Object data)
  {
    MDDCacheKey temp = lookupKey.initialize (1, oper, m1, null, null, data);
    //watch.resume ();
    MDDNode result = (MDDNode)cache.get (temp);
    //watch.pause ();
    return result;
  }
  
  /** find unary operation result.
      @param m1 operand
      @param oper op-code for operation
      @result if the desired result is in the cache, return it;
      otherwise raise a CacheMissException.
  */
  public MDDNode find(MDDNode m1, int oper) 
  {
    //MDDCacheKey temp = new MDDCacheKey (m1, oper);
    MDDCacheKey temp = lookupKey.initialize (1, oper, m1, null, null, null);
    //watch.resume ();
    //log.debug ("Checking for " + temp);
    MDDNode result = (MDDNode)cache.get(temp);
    //watch.pause ();
    return result;
  }

  public MDDNode find (MDDNode m1, MDDNode m2, int oper) {
    //MDDCacheKey temp = new MDDCacheKey (m1, m2, oper);
    MDDCacheKey temp = lookupKey.initialize (2, oper, m1, m2, null, null);
    //log.debug ("Checking for " + temp);
    //if (cache.containsKey (temp))
    //log.debuglog.debug ("HIT!");
    //watch.resume ();
    MDDNode result = (MDDNode)cache.get (temp);
    //watch.pause ();
    return result;
  }
    
  public void insert (MDDNode m1, int oper, Object data, MDDNode result)
  {
    //insertWatch.resume ();
    MDDCacheKey temp = new MDDCacheKey (m1, oper, data);
    cache.put (temp, result);
    //insertWatch.pause ();
  }
  
  public void insert (MDDNode m1, int oper, MDDNode result) {
    //insertWatch.resume ();
    MDDCacheKey temp = new MDDCacheKey (m1,  oper);
    //log.debug ("Trying to insert " + temp + "=" + result);

    cache.put (temp, result);
//      if (!cache.containsKey (temp))
//        cache.put (temp, result);
    //insertWatch.pause();
  }

  public void insert(MDDNode m1, MDDNode m2, int oper, MDDNode result) 
  {
    //insertWatch.resume ();
    MDDCacheKey temp = new MDDCacheKey (m1, m2,  oper);
    cache.put (temp, result);
//      if (!cache.containsKey (temp)) 
//        {
//  	//log.debug ("Not in cache, inserting");
//  	cache.put (temp, result);
//        }
//      else 
//        //("Already cached.");  
    //insertWatch.pause ();
  }

  public void cleanup()
  {
    // eventually do some cleaning-up
    cache = new Hashtable ();
  }

  public void resetStats ()
  {
    watch = new StopWatch ();
    watch.pause ();
    insertWatch = new StopWatch ();
    insertWatch.pause ();
  }

  public String toString ()
  {
    return "Time spend in cache.find: " + watch.toString () + 
      "\nTime spend in cache insert: " + insertWatch.toString ();
  }
  
  public static void main(String argv[])
  {
//      MDDCacheStub c = new MDDCacheStub();
//      MDDNode m1, m2, m3, m4, m5;
//      m1 = new MDDNode(null, 2, 3);
//      m2 = new MDDNode(null, 2, 2);
//      MDDNode chlist[] = {m1, m2, m2, m1};
//      m3 = new MDDNode(null, 0, chlist);
//      MDDNode chlist2[] = {m2, m2, m1, m1};
//      m4 = new MDDNode(null, 0, chlist2);
//      MDDNode chlist3[] = {m3, m4, m4, m3};
//      m5 = new MDDNode(null, 1, chlist3);

//      c.insert(m1, 1, m2);
//      if (c.find(m1, 1) == null) {
//        System.out.println("Error!");
//      }
//      else System.out.println((c.find(m1, 1)).toString());
//      if (c.find(m2, 2) != null)
//        System.out.println("Error!");
  }

  // -- The cache key
  static class MDDCacheKey 
  {
    //static Logger log = new Logger (MDDCacheKey.class.getName ());
  
    MDDNode op1;
    MDDNode op2;
    MDDNode op3;
    int arity;
    int opcode;
    Object data = null; // -- any aditional data
    
    public MDDCacheKey ()
    {
    }
    
    
    public MDDCacheKey (MDDNode op, int unaryOpcode, Object _data)
    {
      this (op, unaryOpcode);
      data = _data;
    }
    
    public MDDCacheKey (MDDNode op, int unaryOpcode) 
    {
      arity = 1;
      opcode = unaryOpcode;
      op1 = op;
      op2 = null;
      op3 = null;
    }

    /** Cache key constructor for binary operations, of the form
	<i>opr1 (op) opr2</i>.
	@param opr1 left operand
	@param opr2 right operand
	@param binaryOpcode  */

    public MDDCacheKey (MDDNode opr1, MDDNode opr2, int binaryOpcode) 
    {
      arity = 2;
      opcode = binaryOpcode;
      op1 = opr1;
      op2 = opr2;
      op3 = null;
    }

    MDDCacheKey initialize (int _arrity, int op, 
			    MDDNode opr1, MDDNode opr2, MDDNode opr3, 
			    Object _data)
    {
      arity = _arrity;
      opcode = op;
      op1 = opr1;
      op2 = opr2;
      op3 = opr3;
      data = _data;
      return this;
    }
    
    
    
    /** Equality test, inherited from Object
	@see java.lang.Object#equals */      
    public boolean equals (Object o)
    {
      if (! (o instanceof MDDCacheKey)) return false;
      
      MDDCacheKey rhs = (MDDCacheKey)o;
      
      // -- check that opcodes and arrities coinside
      if (opcode != rhs.opcode || arity != rhs.arity)
	return false;

      if (data == null && rhs.data != null) return false;
      else if (data != null && !data.equals (rhs.data)) return false;
      

      // -- based on arity check that operands are equal
      switch (arity)
	{
	case 3:
	  if (!op3.equals (rhs.op3)) return false;
	case 2:
	  if (!op2.equals (rhs.op2)) return false;
	case 1:
	  return op1.equals (rhs.op1);
	default:
	  // XXX Need Assertions in edu.toronto.cs.util
	  throw new RuntimeException ("Unknown arity: " + arity);
	}
    }
    
    public int hashCode ()
    {
      long hash = 0;
      switch (arity)
	{
	case 3:
	  hash += op3.objectHashCode () * MDDNode.primes [2];
	case 2:
	  hash += op2.objectHashCode () * MDDNode.primes [1];
	case 1: 
	  hash += (op1.objectHashCode () + opcode) * MDDNode.primes [0];
	}
      if (data != null) 
	hash += data.hashCode ();
      return (int)hash;
    }
    
    public int __hashCode() 
    {
      // XXX is this a good hash function? Why is it like this?
      if (arity == 1)
	return op1.hashCode() + opcode;
      else if (arity == 2)
	return (op1.hashCode() + 3 * op2.hashCode() + opcode);
	else return (op1.hashCode() + 3 * op2.hashCode() +
		     5 * op3.hashCode() + opcode);
    } 

    public String toString() 
    {
      if (arity == 1)
	return "<"+ opcode + "," + op1 + ">";
      else if (arity == 2)
	return "<" + opcode + "," + op1 + "," + op2 + ">";
      else //  (arity == 3)
	return "<" + opcode + "," + op1 + "," + op2 + "," + op3 + ">";
    }
  }


}  // end MDDCacheStub







