package edu.toronto.cs.mdd;

import edu.toronto.cs.algebra.Algebra;
import edu.toronto.cs.mvset.MvState;
import edu.toronto.cs.util.*;

import edu.toronto.cs.mvset.*;


/** The interface for an MDD manager. It is subclassed by
    MDDMgr, HMDDMgr, MBTDDMgr. */
public interface MDDManager 
{

  void setCaching (boolean v);
  int getNumVars ();
  
  /** Return one of the constant nodes. */
  MDDNode getConstant(int val);

  /** Get the "zero", the identity of the "addition" */
  MDDNode getZero();

  /** Get the "one", the identity of the "multiplication" */
  MDDNode getOne();

  /** Return the one-variable node x_i */
  MDDNode getOneVar(int idx);

  /** Return the one-variable node x_i = v */
  MDDNode getVarEquals(int idx, int val);

  /** set an entire cube equal to a value */
  MDDNode setEqualTo(MDDNode m, int val);

  /** Make a unique node */
  MDDNode makeUnique(int varidx, MDDNode[] chdrn);

  /** Unary function application to an MDD */
  MDDNode Apply(MDDNode m1, int oper);

  /** Binary function application */
  MDDNode Apply(MDDNode m1, MDDNode m2, int oper); 

  MDDNode apply (MDDNode m1, MDDNode m2, MDDMgr.ApplyFunction function);
  
  /** Restriction of a variable to a value */
  MDDNode Restrict(MDDNode m, int varidx, int value);

  MDDNode existAbstract (MDDNode node, MDDNode cube);
  MDDNode forallAbstract (MDDNode node, MDDNode cube);
  
  /** Multiple restriction.
      @author cdtw */
  MDDNode applyState(MDDNode m, MvState state);

  /** MvState manufacturing */
  public MvState newState();

  /** Returns an Algebra used by this MDDManager*/
  Algebra getAlgebra ();

  /** Ternary function application */
  //    public MDDNode Apply(MDDNode m1, MDDNode m2, MDDNode m3, int oper);

  // public MDDNode Build(ArbArray aa);

  /** For a given operation <i>oper</i>, iterate it
      over all possible assignments to variable <i>varidx</i>.
      This operation generalizes quantification in a variable.
      @param m1 an MDDNode
      @param varidx a variable
      @param oper an op-code
  */
  MDDNode BigOp(MDDNode m1, int varidx, int oper);

  MDDNode renameVars (MDDNode mddNode, int[] newVars);

  // -- number of nodes in the diagram
  int dagSize (MDDNode mddNode);
  // -- number of nodes in a set of diagrams
  int sharedSize (MDDNode[] mddNode);

  MDDNode transfer (MDDManager dstManager, MDDNode srcNode, 
		    int[] logicMap, int factor);
  MDDNode transfer (MDDManager dstManager, MDDNode srcNode, int[] logicMap);
  MvSet transfer (MvSetFactory dstFactory, MDDNode srcNode, 
		    int[] logicMap, int factor);
  MvSet transfer (MvSetFactory dstFactory, MDDNode srcNode, int[] logicMap);

  MDDNode reduce (MDDNode node, int src, int dst);
  MDDNode killEdge (MDDNode node, boolean[] map, int val);

  void resetStats ();

  MDDPoint getRandomPoint (MDDNode node, ChaosIterator varDomain);
}

    
