package edu.toronto.cs.modelchecker;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.algebra.*;

import java.util.*;

/**
 ** An abstract model reader.
 **/
public abstract class AbstractModelReader
{
  // various options
  protected Map options;

  // the xkripke structure(s)
  protected XKripke[] xkripke = null;

  // the propositional ordering
  protected PropOrderIf propOrder;

  // the resulting transition relation
  protected MvSet transRelation;

  // initial state
  protected MvState initState;

  // -- enumerated types that we know about
  protected EnumeratedTypes enumTypes = EnumeratedTypes.EMPTY_TYPES;

  public void setOptions (Map opts)
  {
    options = opts;
  }

  public abstract void makeTransRelation (MvSetFactory ddManager);

  /**
   ** Get the xkripke structure(s).
   ** @return xkripke structure(s).
   **/
  public XKripke [] getXKripke ()
  {
    return xkripke;
  }

  /**
   ** Get the propositional ordering.
   ** @return propositional ordering.
   **/
  public PropOrderIf getPropOrder ()
  {
    return propOrder;
  }

  
  /**
   ** Get the transition relation.
   ** @return transition relation.
   **/
  public MvSet getTransRelation ()
  {
    // this is kind of bad style, but we can be reasonably sure that
    // nothing is going to happen before a transition relation is
    // defined

    // set up the default model name if no other name has been specified
    if (options.get ("model name") == null)
      options.put ("model name", ((List) options.get ("input")).get (0));

    return transRelation;
  }

  /**
   ** Get the initial state.
   ** @return initial state.
   **/
  public MvState getInitState ()
  {
    return initState;
  }

  /**
   ** Get the enumerated types we know about.
   ** @return enumerated types we know about.
   **/
  public EnumeratedTypes getEnumTypes ()
  {
    return enumTypes;
  }

  /**
   ** Get the lattice.
   ** @return lattice.
   **/
  public abstract FiniteLattice getLattice ();

}
