package edu.toronto.cs.modelchecker;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.xkripke.XKripke.*;
import edu.toronto.cs.util.*;

import java.util.*;

/****
 **** Implements alternating ordering so that each proposition 'p' is mapped 
 **** to two MDD variables 'x_i' and 'x_{i+1}' where 'p' is 'x_i' and 
 **** 'p`' is 'x_{i+1}'
 ****/
public class AlternatePropOrder implements PropOrderIf
{
  static Logger log = new Logger ("AlternatePropOrder");
  String[] propNames;
  
  public AlternatePropOrder (String[] _propNames)
  {
    log.debug ("Initializing AlternatePropOrder with: " + 
	       Arrays.asList (_propNames));
    System.out.println ("AltOrder: \n" + Arrays.asList (_propNames));
    propNames = _propNames;
  }
  
  public Iterator orderTransition (XKripkeTransition transition)
  {
    final  XKripkeTransition trans = transition;
    return new Iterator ()
      {
	int cur = 0;
	public boolean hasNext ()
	  {
	    return cur < size ();
	  }
	public Object next ()
	  {
	    // -- pick the appropriate state
	    log.debug 
	      ("On trans "+trans.getSrc ()+" to "+trans.getDst ());
	    XKripkeState state = 
	      isPreImage (cur) ? trans.getSrc () : trans.getDst ();
	    log.debug ("Chose state: "+state);
	    // -- return the value and increment cur
	    log.debug 
	      ("Fetching (for trans-rel init): "+getPropName (cur));
	    return state.getProp (getPropName (cur++));	    
	  }
	public void remove (){}
      };
  }
  

  public int getPropOrder (String name)
  {
    for (int i = 0; i < propNames.length; i++)
      if (name.equals (propNames [i]))
	return i << 1;
    return -1;
  }

  public int getNextPropOrder (String name)
  {
    for (int i = 0; i < propNames.length; i++)
      if (name.equals (propNames [i]))
	return (i << 1) | 1;
    return -1;
  }
  
  int[] primedMap = null;
  
  /****
   **** Computes a map mapping each un primed prop to a primed one
   **** In this particular case that is a map that is an identity map on 
   **** odd numbers and is x -> x + 1 on even numbers*/
  public int[] getPrimedMap ()
  {
    if (primedMap == null)
      {
	primedMap = new int [size ()];
	for (int i = 0; i < primedMap.length - 1; i += 2)
	  {
	    primedMap [i] = i + 1;
	    primedMap [i + 1] = i + 1;
	  }
      }
    
    return primedMap;
  }

  int[] unPrimedMap = null;
  public int[] getUnPrimedMap ()
  {
    if (unPrimedMap == null)
      {
	unPrimedMap = new int [size ()];
	for (int i = 0; i < unPrimedMap.length - 1; i += 2)
	  {
	    unPrimedMap [i] = i;
	    unPrimedMap [i + 1] = i;
	  }
      }
    
    return unPrimedMap;
  }
  
  int[] primedProps = null;
  int[] unPrimedProps = null;
  
  public int[] getPrimedProps ()
  {
    if (primedProps == null)
      {
	primedProps = new int [propNames.length];
	for (int i = 0; i < primedProps.length; i++)
	  // -- 2 * i + 1
	  primedProps [i] = ((i << 1) | 1);
      }    
    return primedProps;
  }

  public int[] getUnPrimedProps ()
  {
    if (unPrimedProps == null)
      {
	unPrimedProps = new int [propNames.length];
	for (int i=0; i < unPrimedProps.length; i++)
	  unPrimedProps [i] = i << 1; 
      }
    return unPrimedProps;
  }
  
  public String getPropName (int idx)
  {
    // -- this is integer division by 2
    return propNames [idx >> 1];
  }
  

  public boolean isPreImage (int idx) 
  {
    return getPropType (idx) == PropOrderIf.PRE_IMAGE;
  }
  
  public boolean isPostImage (int idx)
  {
    return getPropType (idx) == PropOrderIf.POST_IMAGE;
  }
  
  public int getPropType (int idx)
  {
    // -- need to implement assertions
    // Assert.assert (idx >= 0 && idx < size ());
    
    // -- even ids are pre images, odd are post images
    return ((idx & 1) == 0) ? PropOrderIf.PRE_IMAGE : PropOrderIf.POST_IMAGE;
  }
  

  public int size ()
  {
    /// -- propNames * 2
    return propNames.length << 1;
  }
  
}






