package edu.toronto.cs.mvset;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.util.IntIterator;
import edu.toronto.cs.modelchecker.PropOrderIf;

public abstract class AbstractMvSet implements MvSet 
{
  public abstract MvSet ptwiseCompose (int op, MvSet g);

  /***
   *** Creates a ptwise negation
   *** h (x) = \neg f (x)
   ***/
  public abstract MvSet ptwiseNeg ();

  /***
   *** Ptwise compare: <=, >=, =.
   ***/
  public abstract MvSet ptwiseCompare (int op, MvSet g);

  /***
   *** Restricts an argument
   *** h (x_0, x_1, x_2, ...) = f (x_0, ..., value, ...)
   *** where value is substituted at argIdx
   ***/
  public abstract MvSet restrict (int argIdx, int value);

  /***
   *** Restricts the MvSet to a particular state (or sets of states if 
   *** 'state' is partial)
   ***/
  public abstract MvSet restrict (MvState state);

  public MvSet existAbstract (MvSet cube)
  {
    System.out.println ("Bad existAbstract");
    edu.toronto.cs.util.Assert.assert ("Hmm, not implemented!");
    return null;
  }
  
  
  public MvSet existAbstract (int[] varIndex)
  {
    MvSet result = this;
    
    for (int i = 0; i < varIndex.length; i++)
      result = result.existAbstract (varIndex [i]);
    return result;
  }
  
  public MvSet existAbstract (int var)
  {
    FiniteLattice lattice = getLattice ();
    
    MvSet result = getFactory ().createConstant (lattice.idplusIdx ());
    for (IntIterator it = lattice.carrierSet (); it.hasNext ();)
      result = result.or (this.restrict (var, it.nextInt ()));
    return result;
  }

  public MvSet forallAbstract (MvSet cube)
  {
    System.out.println ("Bad forallAbstract");
    edu.toronto.cs.util.Assert.assert ("Hmm, not implemented!");
    return null;
  }
  
  
  public MvSet forallAbstract (int[] varIndex)
  {
    MvSet result = this;
    
    for (int i = 0; i < varIndex.length; i++)
      result = result.forallAbstract (varIndex [i]);
    return result;
  }
  
  public MvSet forallAbstract (int var)
  {
    FiniteLattice lattice = getLattice ();
    
    MvSet result = getFactory ().createConstant (lattice.idplusIdx ());
    for (IntIterator it = lattice.carrierSet (); it.hasNext ();)
      result = result.and (this.restrict (var, it.nextInt ()));
    return result;
  }

  
  /***
   *** Renames the arguments. newArgs is a map from old args to new so that
   *** h (x) = f (newArgs [0], newArgs [1], ...)
   ***/
  public abstract MvSet renameArgs (int[] newArgs);
  
  /***
   *** Evaluates this function on an input
   *** result = f (values [0], values [1], ...)
   ***/
  public abstract int evaluate (int[] values);

  /***
   *** Evaluates this function with respect to a state
   *** Note that we have a small discreptancy between what we mean 
   *** by states since here a state may contain primed and unprimed variables
   *** and thus represent a transition rather than a state
   ***/
  public abstract int evaluate (MvState state);


  /*** arrity of this function */
  public abstract int getArrity ();

  /*** gets the factory */
  public abstract MvSetFactory getFactory ();

  /*** gets the lattice for the MvSet */
  public abstract FiniteLattice getLattice ();

  /*** Utility functions */
  public MvSet and (MvSet v)
  {
    return ptwiseCompose (getLattice ().MEET, v);
  }
  
  public MvSet or (MvSet v)
  {
    return ptwiseCompose (getLattice ().JOIN, v);
  }
  
  public MvSet not ()
  {
    return ptwiseNeg ();
  }
  
  public MvSet leq (MvSet v)
  {
    return ptwiseCompare (getLattice ().BELOW, v);
  }
  
  public MvSet geq (MvSet v)
  {
    return ptwiseCompare (getLattice ().ABOVE, v);
  }
  
  public MvSet eq (MvSet v)
  {
    return ptwiseCompare (getLattice ().EQUAL, v);
  }

  public void reorder ()
  {
    return;
  }
  
  public int size ()
  {
    return 0;
  }

  public void resetStats ()
  {
  }

  public void writeToDaVinci (String filename, PropOrderIf propOrder)
  {
    System.out.println ("Not implemented here!");
  }
  

  

}
