package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.ctl.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.keg.*;

public class SensibleCTLToStringVisitor extends CTLVisitorDefaulterAncestor
{
  //  MvSet state;
  //StringBuffer s;
  Map nodesToStrings;
  //  FiniteLattice lattice;
  CTLVisitor nodeNamer;


  protected SensibleCTLToStringVisitor ()
  {
    //state = _state;
    //lattice = _lattice;
    //s = new StringBuffer ();
    nodesToStrings = new HashMap (); // Hmm... perhaps *not* the best
				     // choice?
    nodeNamer = new FairNodeNamer ();
  }

  protected StringBuffer getStringBuffer (CTLNode n)
  {
    return (StringBuffer) nodesToStrings.get (n);
  }

  protected void doForEveryNode (CTLNode n, Object stateinfo)
  {
  }

  public static String getSensibleString (CTLNode n, 
					  Object stateinfo)
  {
    SensibleCTLToStringVisitor v = new SensibleCTLToStringVisitor ();
    v.visit (n, stateinfo);
    return v.getStringBuffer (n).toString ();
  }
  
  public void visitAbstractNode (CTLAbstractNode node, Object stateinfo)
  {
    // er....
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append ("-");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);  
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitAFNode (CTLAFNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitAGNode (CTLAGNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("(");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append (")");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitAUNode (CTLAUNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("[");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" U ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append ("]");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitAUiNode (CTLAUiNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("[");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" U ");
    sb.append ("" + node.getI () + " ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append ("]");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitAWNode (CTLAWNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("[");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" W ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append ("]");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitAXNode (CTLAXNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitAndNode (CTLAndNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    sb.append ('(');
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" /\\ ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append (')');
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitAtomNode (CTLAtomNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    //    sb.append ("A[ ");
    //sb.append (node.getLeft ());
    //sb.append (" U ");
    sb.append (node.getName ());
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  }

  public void visitEFNode (CTLEFNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitEGNode (CTLEGNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("(");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append (")");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitEUNode (CTLEUNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("[");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" U ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append (" ]");
    nodesToStrings.put (node, sb);
  doForEveryNode (node, stateinfo);  }

  public void visitEUiNode (CTLEUiNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("[");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" U ");
    sb.append (""+node.getI ()+ " ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append ("]");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitEWNode (CTLEWNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append ("[");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" W ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append ("]");
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);
  }


  public void visitEXNode (CTLEXNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    StringBuffer name = new StringBuffer ();
    nodeNamer.visit (node, name);
    sb.append (name);
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitEqualsNode (CTLEqualsNode node, Object stateinfo)  
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    //    sb.append ("A[ ");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" = ");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitIffNode (CTLIffNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    //    sb.append ("A[ ");
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" <-> ");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitImplNode (CTLImplNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" -> ");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitLatticeNode (CTLLatticeNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    //  sb.append (nodesToStrings.get (node.getLeft ()));
//      sb.append (" -> ");
    sb.append (node.toString ());
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitMvSetNode (CTLMvSetNode node, Object stateinfo)
  {
    // Ah! Fun!
    StringBuffer sb = new StringBuffer ();
    List ancestors = (List)stateinfo;
    MvSet s = node.getMvSet ();
    int i = 0;
    int j = 0;
    
    for (Iterator it = ancestors.iterator (); it.hasNext ();)
      {
	PrettyKegGraphNode kn = (PrettyKegGraphNode) it.next ();
	if (kn.getKegNode () instanceof KEGStateNode)
	  i++;
	else
	  continue;
	
	int v = s.evaluate (((KEGStateNode)kn.getKegNode ()).getState ());
	
	if (((KEGStateNode)kn.getKegNode ())
	    .getState ().getLattice ().idtimesIdx () == v)
	  {
	    j = i;
	    break;
	  }
      } 

    sb.append ("{");
    if (j != 0)
      sb.append ("the ");
    sb.append (StringUtil.ordinate (j, "this"));
    sb.append (" state");
    if (j != 0)
      sb.append (" back.");
    else
      sb.append ('.');
    sb.append ('}');
    //System.out.println (sb.toString ());
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  } 

  public void visitNegNode (CTLNegNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    //  sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append ("!");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);
  }


  public void visitNode (CTLNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    // sb.append (nodesToStrings.get (node.getLeft ()));
    // sb.append (" -> ");
    sb.append (node.toString ());
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);
  }

  
  public void visitOrNode (CTLOrNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    sb.append ('(');
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" \\/ ");
    sb.append (nodesToStrings.get (node.getRight ()));
    sb.append (')');
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitOverNode (CTLOverNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" >= ");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitPlaceholderNode (CTLPlaceholderNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    //  sb.append (nodesToStrings.get (node.getLeft ()));
//      sb.append (" -> ");
    sb.append (node.toString ());
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }

  public void visitPreEXNode (CTLPreEXNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    sb.append ("Pre-EX ");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo);  
  }
  
  public void visitUnderNode (CTLUnderNode node, Object stateinfo)
  {
    visit (node.getLeft (), stateinfo);
    visit (node.getRight (), stateinfo);
    StringBuffer sb = new StringBuffer ();
    sb.append (nodesToStrings.get (node.getLeft ()));
    sb.append (" <= ");
    sb.append (nodesToStrings.get (node.getRight ()));
    nodesToStrings.put (node, sb);
    doForEveryNode (node, stateinfo); 
  }

    

  
}
