package edu.toronto.cs.smv;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.tlq.*;


import java.util.*;
import java.io.*;


public class SMVBddModule extends SMVModule
{
  String bddFileName;
  String initFileName;
  String evarsFileName;
  
  EnumeratedTypes etypes;
  
  public SMVBddModule (String[] _vars, String _bddFileName,
		       String _initFileName, String _evarsFileName)
  {
    super ("");
    vars = _vars;
    bddFileName = _bddFileName;
    initFileName = _initFileName;
    evarsFileName = _evarsFileName;
    
    etypes = null;
    setModelName (_bddFileName);
  }

  public EnumeratedTypes getEnumTypes() 
  {
    if (etypes == null)
      getEtypesFromFile();
    
    return etypes;
    
    
  }
  
  void getEtypesFromFile() 
  { 
    System.out.println("Initializing enumerated types");
    
    etypes = new SimpleEnumTypes();
    
    try {
      
      BufferedReader in = new BufferedReader (new FileReader (evarsFileName));
    while (in.ready())
      {
	StringTokenizer st = new StringTokenizer(in.readLine(),
						 ":");
	if (st.countTokens() == 2) {
	  
	// assignments appear on each line as "var, value"
	// eg. "coffee,T"
	  String var = st.nextToken();
	  String rg = st.nextToken();
	  StringTokenizer st2 = new StringTokenizer(rg, ",");
	  String range[] = new String[st2.countTokens()];
	  int i = 0;
	  while (st2.hasMoreTokens()) 
	    range[i++] = st2.nextToken();
	  etypes.addVariable(var, range);
	}
	// if not 2 tokens, ignored the line: malformed.
      }
    
    }
    catch (IOException ex)
      {
	Assert.assert (ex);
      }
    
   
  }
  
  

  public MvSet computeTrans ()
  {
    return DDUnDumper.readBDDSafe (ddManager, bddFileName);
  }
  
  public MvState initializeInitState (MvState initState)
  {
    // XXX This needs work

    try {
      
    BufferedReader in = new BufferedReader (new FileReader (initFileName));
    while (in.ready())
      {
	StringTokenizer st = new StringTokenizer(in.readLine(),
						 ",");
	if (st.countTokens() == 2) {
	  
	// assignments appear on each line as "var, value"
	// eg. "coffee,T"
	String stateVar = st.nextToken();
	String varVal = st.nextToken();
	System.out.println("initial state: "+stateVar+"="+varVal);
	if (varIndex == null)
	  System.out.println ("var index null");
	if ((Integer)varIndex.get(stateVar) == null)
	  System.out.println ("index entry null");
	initState.setValue(getVarIndex(stateVar),
			   lattice.getElementIdxByKey(varVal));
	
	}
	// if not 2 tokens, ignored the line: malformed.
      }
    
    }
    catch (IOException ex)
      {
	Assert.assert (ex);
      }
    
    System.out.println(initState.toString(ordering));
    
    return initState;
  }
  
  
  public static SMVModule initModule (String[] args)
  {
    /***
	args [0] -- prop ordering file
	args [1] -- bdd file
	args [2] -- initial state file
	args [3] -- enumerted types file

    ***/
    //    String latticeName = args [0];
    
    String propOrderFileName = args [0];
    String bddFileName = args [1];
    String initFileName = args[2];
    String etypesFileName = args [3];
        
    String[] vars = computeVars (propOrderFileName);

    
    SMVBddModule module = new SMVBddModule (vars, bddFileName,
					    initFileName, etypesFileName);
    try {
      //module.setLattice(LatticeFactory.parse("play/gxl/2val-logic.xml"));
      module.setLattice(new UpSetLattice());
      
    }
    catch (Exception e) {
      System.out.println(e);
      System.exit(-1);
    }
     
    
    
    return module;
  }
  

  // -- reads variables from a file
  public static String[] computeVars (String fileName)
  {
    System.out.println("SBM: computeVars started");
    
    try 
      {
	List vars = new ArrayList ();
	BufferedReader in = new BufferedReader (new FileReader (fileName));
	String s;
	while ( (s = in.readLine ()) != null) 
	  	  vars.add (s);
	
	return (String[])vars.toArray (new String [vars.size ()]);
      }
    catch (IOException ex)
      {
	Assert.assert (ex);
      }
    return null;
  }
  
}
