package edu.toronto.cs.util;

import java.util.StringTokenizer;


/***
 *** Parses a stream and fixes some HTML tags to be in correspondence
 *** with XML
 *** namely: 
 *** <br> --> <br></br>
 *** <p>text<p> --> <p>text</p><p>
 *** <p>text EOF --> <p>text</p>EOF
 ***/
public class HtmlFixer
{

  public static int TEXT = 1;
  public static int TAG_START = 2;
  
  public static String fixString (String s)
  {

    // -- that's where the output will go
    StringBuffer sb = new StringBuffer ();

    
    // -- true if we have an open paragraph
    boolean openParagraph = false;

    StringTokenizer st = new StringTokenizer (s, "<>", true);
    
    int mode = TEXT;
    while (st.hasMoreTokens ())
      {
	String token = st.nextToken ();
	if (token.charAt (0) == '<')
	  {
	    mode = TAG_START;
	    continue;
	  }
	else if (token.charAt (0) == '>')
	  {
	    mode = TEXT;
	    continue;
	  }
	
	if (mode == TEXT)
	  {
	    sb.append (token);
	    continue;
	  }
	
	// -- mode is TAG_START
	if (token.equals ("/p"))
	  openParagraph = false;
	else if (token.equals ("p"))
	  {
	    // -- if a paragraph is open, close it and start the new one
	    if (openParagraph)
	      sb.append ("</p>");
	    
	    sb.append ("<p>");
	    openParagraph = true;
	  }
	else if (token.equalsIgnoreCase ("br"))
	  sb.append ("<br></br>");
	else // -- some other tag
	  sb.append ("<" + token + ">");
	
      }
    
    
    // -- if we are done but a paragraph is still open, then close it
    if (openParagraph)
      sb.append ("</p>");

    return sb.toString ();
  }

  public static void main (String[] args)
  {
    System.out.println ("Input string:\n" + args [0]);
    System.out.println ("Output string:\n" + fixString (args [0]));
    
  }
  

}
