package edu.toronto.cs.util.dgraph;

import java.util.*; // We all love java.util ...
import edu.toronto.cs.util.*;

public class DGSinglePass extends DGraphVisitorAncestor
{
  static Logger log = new Logger ("DGSinglePass");

  DGraphVisitor visitor;
  Set visited;

  // If true, visit nodes even if they're already in the visited set
  // (just don't traverse their children.)
  boolean visitTwice = false;

   static public DGraphVisitor traverseWith (DGraph graph, DGraphVisitor visitor)
  {
    DGSinglePass dgs = new DGSinglePass (visitor);
    dgs.traverse (graph);
    return dgs.getVisitor ();
  }
  
  protected DGSinglePass (DGraphVisitor _visitor)
  {
    visitor = _visitor;
    visited = new HashSet ();
  }

  protected void traverse (DGraph graph)
  {
    Iterator firstnodes = graph.getRootNodes ().iterator ();
    while (firstnodes.hasNext ())
      {
	log.debug ("Testing rootnode: ");
	DGNode nnode = (DGNode) firstnodes.next ();
	//log.debug (nnode);
	if (!visited.contains (nnode))
	  {
	    log.debug ("   Visiting "+nnode);
	    visited.add (nnode);
	    traverse (nnode);
	  }
	else if (visitTwice)
	  {
	    //visitor.visit (nnode, null);
	    applyVisitor (nnode);
	  }
      }
  }

  public void traverse (DGNode node)
  {
    //visitor.visit (node, null);
    applyVisitor (node);
    log.debug ("Testing children");
    for (Iterator it = node.getSuccessors ().iterator (); it.hasNext ();)
      {
	DGNode nnode = (DGNode) it.next ();
	if (!visited.contains (nnode))
	  {
	    log.debug ("   Visiting "+nnode);
	    visited.add (nnode);
	    traverse (nnode);
	  }
	else if (visitTwice)
	  {
	    //visitor.visit (nnode, null);
	    applyVisitor (nnode);
	  }
	
      }
  }

  protected void applyVisitor (DGNode nnode)
  {
    visitor.visit (nnode, null);
  }
  
  public DGraphVisitor getVisitor ()
  {
    return visitor;
  }
  
}

  
